package org.processmining.models.operationalsupport.gui;

import java.awt.Component;
import java.util.Map.Entry;
import java.util.Vector;

import javax.swing.JTree;

import org.processmining.models.operationalsupport.Analysis;
import org.processmining.models.operationalsupport.Attributes;
import org.processmining.models.operationalsupport.Prediction;
import org.processmining.models.operationalsupport.ProviderResponse;
import org.processmining.models.operationalsupport.Recommendation;
import org.processmining.models.operationalsupport.Response;

public class ResponseVisualizer {

	private static final String ANALYSIS = "Analysis";
	private static final String PREDICTION = "Prediction";
	private static final String RECOMMENDATION = "Recommendation";

	private final String analysis;
	private final String prediction;
	private final String recommendation;

	public ResponseVisualizer() {
		super();
		analysis = ANALYSIS;
		prediction = PREDICTION;
		recommendation = RECOMMENDATION;
	}

	public ResponseVisualizer(String analysis, String prediction, String recommendation) {
		super();
		this.analysis = analysis;
		this.prediction = prediction;
		this.recommendation = recommendation;
	}

	public Component getVisualizationPanel(Response response) {
		Vector<NamedVector<NamedVector<OSNode>>> rootNodes = new NamedVector<NamedVector<NamedVector<OSNode>>>(
				"Response for request nr. " + response.getrequestId());
		for (ProviderResponse pr : response.getProviderResponses()) {

			NamedVector<OSNode> analysis = new NamedVector<OSNode>(this.analysis);
			for (Analysis a : pr.getAnalysis()) {
				analysis.add(createAnalysisNode(a));
			}

			NamedVector<OSNode> prediction = new NamedVector<OSNode>(this.prediction);
			for (Prediction a : pr.getPrediction()) {
				prediction.add(createAnalysisNode(a));
			}

			NamedVector<OSNode> recommendation = new NamedVector<OSNode>(this.recommendation);
			for (Recommendation a : pr.getRecommendation()) {
				Recommendation rec = new Recommendation(RECOMMENDATION, a.getRecItems());
				recommendation.add(createRecommendationNode(a, rec));
			}

			NamedVector<NamedVector<OSNode>> provider = new NamedVector<NamedVector<OSNode>>(pr.getProviderName());
			if (!analysis.isEmpty()) {
				provider.add(analysis);
			}
			if (!prediction.isEmpty()) {
				provider.add(prediction);
			}

			if (!recommendation.isEmpty()) {
				provider.add(recommendation);
			}

			if (!(analysis.isEmpty() && prediction.isEmpty() && recommendation.isEmpty())) {
				rootNodes.add(provider);
			}
		}
		JTree tree = new JTree(rootNodes);
		tree.setCellRenderer(new OSTreeCellRenderer());
		expandAll(tree);
		return tree;
	}

	/**
	 * Expands all nodes in the tree
	 */
	private void expandAll(JTree tree) {
		int row = 0;
		while (row < tree.getRowCount()) {
			tree.expandRow(row);
			row++;
		}
	}

	private OSNode createAnalysisNode(Analysis a) {
		Iterable<Entry<String, Attributes>> attributes = a.getAllAttributes();
		if (!a.getAllAttributes().iterator().hasNext()) {
			return new EmptyNode(a);
		} else {
			NodeWithAttributes node = new NodeWithAttributes(a);
			for (Entry<String, Attributes> entry : attributes) {
				node.add(new HTMLAttributes(entry.getKey(), entry.getValue()));
			}
			return node;
		}
	}

	private OSNode createRecommendationNode(Analysis analysis, Recommendation recommendation) {
		NodeWithAttributes node = new NodeWithAttributes(analysis, recommendation);
		node.add(new HTMLAttributes(RECOMMENDATION, recommendation));
		return node;
	}

	private class NamedVector<T> extends Vector<T> {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		String name;

		public NamedVector(String name) {
			this.name = name;
		}

		//		public NamedVector(String name, T elements[]) {
		//			this.name = name;
		//			for (int i = 0, n = elements.length; i < n; i++) {
		//				add(elements[i]);
		//			}
		//		}

		public String toString() {
			return name;
		}
	}

}
