package org.processmining.models.operationalsupport.net.client;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;

import org.processmining.models.operationalsupport.Request;
import org.processmining.models.operationalsupport.Response;
import org.processmining.models.operationalsupport.net.OperationalSupportProtocol;
import org.processmining.models.operationalsupport.xml.OSXMLConverter;

public class OperationalSupportClient {

	protected String host;
	protected int port;
	protected OSXMLConverter converter;

	//
	/**
		 * 
		 */
	public OperationalSupportClient(String aHost, int aPort) {
		host = aHost;
		port = aPort;
		converter = new OSXMLConverter();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.processmining.framework.models.recommendation.RecommendationProvider
	 * #getRecommendation
	 * (org.processmining.framework.models.recommendation.Recommendationrequest)
	 */
	public Response getResponse(Request request) throws Exception, IOException {
		// connect to socket
		Socket socket = new Socket(host, port);
		socket.setKeepAlive(true);
		socket.setSoTimeout(0);
		// connect reader and writer
		BufferedReader in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
		PrintWriter out = new PrintWriter(socket.getOutputStream());
		String line = in.readLine();
		if (line.equals(OperationalSupportProtocol.HELO_SERVER_PROLOG) == false) {
			throw new Exception("Fatal error: Remote is no recommendation server! (HELO mismatch!)");
		}

		boolean receivedId = false;
		int nrTries = 0;
		while (!receivedId && (nrTries < 10)) { // give it 10 tries to receive the ID
			line = in.readLine(); // get the request id
			try {
				nrTries++;
				Long.parseLong(line);
				receivedId = true;
				request.resetId(line);
			} catch (NumberFormatException e) {
				// ignore
			}
		}

		boolean receivedPort = false;
		int newPort = 0;
		nrTries = 0;
		while (!receivedPort && (nrTries < 10)) { // give it 10 tries to receive the port
			line = in.readLine(); // get the request id
			try {
				nrTries++;
				newPort = Integer.parseInt(line);
				receivedPort = true;
			} catch (NumberFormatException e) {
				// ignore
			}
		}

		while ((line = in.readLine()).equalsIgnoreCase(OperationalSupportProtocol.HELO_SERVER_EPILOG) == false) {
			// wait for server HELO epilog so we can start request
		}

		if (!receivedPort) {
			// ERROR
		} else {
			// close connection
			socket.close();
			socket = null;

			// reconnect on new port
			socket = new Socket(host, newPort);
			socket.setKeepAlive(true);
			socket.setSoTimeout(0);
			// connect reader and writer
			in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
			out = new PrintWriter(socket.getOutputStream());

		}

		// write request
		String requestXML = converter.requestToXML(request);

		writeLine(out, OperationalSupportProtocol.CLIENT_REQUEST_request_PROLOG);
		writeLine(out, requestXML);
		writeLine(out, OperationalSupportProtocol.CLIENT_REQUEST_XML_EPILOG);
		// wait for receipt confirmation
		while ((line = in.readLine()).equalsIgnoreCase(OperationalSupportProtocol.SERVER_REQUEST_XML_RECEIVED) == false) {
			// wait for receipt
		}
		// wait for outcome
		while ((line = in.readLine()).equalsIgnoreCase(OperationalSupportProtocol.SERVER_RESPONSE_POSITIVE) == false) {
			if (line.equalsIgnoreCase(OperationalSupportProtocol.SERVER_RESPONSE_NEGATIVE)) {
				// error. print message to stderr...
				String errorMessage = "Remote server error:\n";
				System.err.println("Remote server at " + host + ":" + port + " reported a fatal error:");
				while ((line = in.readLine()).equalsIgnoreCase(OperationalSupportProtocol.SERVER_GOODBYE) == false) {
					System.err.println(line);
					errorMessage += line + "\n";
				}
				socket.close();
				throw new Exception(errorMessage);
			}
		}
		// ok, it seems to have worked..
		// wait for response prolog
		while ((line = in.readLine()).equalsIgnoreCase(OperationalSupportProtocol.SERVER_RESULT_PROLOG) == false) {
			// wait for result prolog
		}
		// collect result message
		StringBuilder resultStringBuilder = new StringBuilder();
		while ((line = in.readLine()).equalsIgnoreCase(OperationalSupportProtocol.SERVER_RESULT_EPILOG) == false) {
			resultStringBuilder.append(line);
			resultStringBuilder.append('\n'); // fix newline char
		}
		// send goodbye
		writeLine(out, OperationalSupportProtocol.CLIENT_GOODBYE);
		// wait for server goodbye
		while ((line = in.readLine()).equalsIgnoreCase(OperationalSupportProtocol.SERVER_GOODBYE) == false) {
			// wait for server goodbye
		}
		// close connection
		socket.close();
		socket = null;
		// unmarshal result and return it
		Response result = converter.responseFromXML(resultStringBuilder.toString());
		return result;
	}

	/**
	 * safe and convenient writing to socket character streams; internal helper
	 * method
	 * 
	 * @param out
	 * @param line
	 */
	protected void writeLine(PrintWriter out, String line) {
		out.write(line + "\r\n");
		out.flush();
	}
}
