package org.processmining.models.operationalsupport.xml;

import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.xstream.XConverter;
import org.processmining.models.operationalsupport.Request;
import org.processmining.models.operationalsupport.WorkItem;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.converters.MarshallingContext;
import com.thoughtworks.xstream.converters.UnmarshallingContext;
import com.thoughtworks.xstream.io.HierarchicalStreamReader;
import com.thoughtworks.xstream.io.HierarchicalStreamWriter;

class XOSRequestConverter extends XConverter {

	private static final String ID = "id";
	private static final String PROCESS_ID = "processId";
	private static final String PROCESS_INSTANCE_ID = "processInstanceId";
	private static final String TRACE = "trace";
	private static final String AVAILABLE = "available";
	private static final String ITEM = "item";

	@SuppressWarnings("unchecked")
	public boolean canConvert(Class clazz) {
		return clazz.equals(Request.class); // convert only instances of OperationalSupportrequest
	}

	public void marshal(Object value, HierarchicalStreamWriter writer, MarshallingContext context) {
		Request request = (Request) value;

		writer.addAttribute(ID, request.getId());
		writer.addAttribute(PROCESS_ID, request.getProcessId());
		writer.addAttribute(PROCESS_INSTANCE_ID, request.getProcessInstanceId());

		writer.startNode(TRACE);
		context.convertAnother(request.getTrace());
		writer.endNode();

		writer.startNode(AVAILABLE); // add analysis
		for (WorkItem a : request.getAvailableItems()) {
			writer.startNode(ITEM);
			context.convertAnother(a);
			writer.endNode();
		}
		writer.endNode();
	}

	public Object unmarshal(HierarchicalStreamReader reader, UnmarshallingContext context) {
		String id = reader.getAttribute(ID);
		String processId = reader.getAttribute(PROCESS_ID);
		String processInstanceId = reader.getAttribute(PROCESS_INSTANCE_ID);

		Request request = new Request(id, processId, processInstanceId);

		while (reader.hasMoreChildren()) {
			reader.moveDown();
			if (TRACE.equals(reader.getNodeName())) {
				XTrace trace = (XTrace) context.convertAnother(request, XTrace.class);
				request.addAllEvents(trace);
			}

			if (AVAILABLE.equals(reader.getNodeName())) { // get analysis items
				while (reader.hasMoreChildren()) {
					reader.moveDown();
					WorkItem a = (WorkItem) context.convertAnother(request, WorkItem.class);
					request.addAvailableItem(a);
					reader.moveUp();
				}
			}
			reader.moveUp();
		}

		/*
		 * if (reader.hasMoreChildren()) { reader.moveDown(); if
		 * (AVAILABLE.equals(reader.getNodeName())) { while
		 * (reader.hasMoreChildren()) { reader.moveDown(); AvailableItem item =
		 * (AvailableItem) context.convertAnother(request, AvailableItem.class);
		 * request.addAvailableItem(item); reader.moveUp(); } } reader.moveUp();
		 * }
		 */
		return request;
	}

	public void registerAliases(XStream stream) {
		super.registerAliases(stream);
		stream.alias("request", Request.class);
	}

}
