package org.processmining.models.operationalsupport.xml;

import org.deckfour.xes.xstream.XConverter;
import org.processmining.models.operationalsupport.ProviderResponse;
import org.processmining.models.operationalsupport.Response;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.converters.MarshallingContext;
import com.thoughtworks.xstream.converters.UnmarshallingContext;
import com.thoughtworks.xstream.io.HierarchicalStreamReader;
import com.thoughtworks.xstream.io.HierarchicalStreamWriter;

class XOSResponseConverter extends XConverter {

	private static final String request_ID = "requestId";
	private static final String PROVIDER = "provider";

	public void marshal(Object value, HierarchicalStreamWriter writer, MarshallingContext context) {
		try {
			Response response = (Response) value;
			writer.addAttribute(request_ID, response.getrequestId());

			for (ProviderResponse pr : response.getProviderResponses()) {
				writer.startNode(PROVIDER);
				context.convertAnother(pr);
				writer.endNode();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public Object unmarshal(HierarchicalStreamReader reader, UnmarshallingContext context) {
		String requestId = reader.getAttribute(request_ID);

		Response response = new Response(requestId);

		while (reader.hasMoreChildren()) {
			reader.moveDown();
			if (PROVIDER.equals(reader.getNodeName())) {
				ProviderResponse pr = (ProviderResponse) context.convertAnother(response, ProviderResponse.class);
				response.add(pr);
			}
			reader.moveUp();
		}
		return response;
	}

	@SuppressWarnings("unchecked")
	public boolean canConvert(Class clazz) {
		return clazz.equals(Response.class);
	}

	public void registerAliases(XStream stream) {
		super.registerAliases(stream);
		stream.alias("response", Response.class);
	}

}
